//
//  deserializer_space.c
//  lymui
//
//  Created by Marc Intha on 24/11/2018.
//  Copyright © 2018 Marc. All rights reserved.
//

#include "deserializer_space.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <node_api.h>
#include "binding_error.h"
#include "binding_util.h"
#include "deserializer_opts.h"

BridgeObj *deserializeSpace(napi_env env, napi_value obj) {
    BridgeObj *br = malloc(sizeof(BridgeObj));
    if (br == NULL) {
        return NULL;
    }
    
    char *inputProps = "input:output";
    napi_value params[2];
    
    if (!hasPropInJSObj(env, obj, inputProps)) {
        br->error = ARG_NB_ERR;
        return br;
    }
    
    char *err = getNamedPropArray(env, inputProps, obj, CONVERT_BASIC_LEN, params);
    if (err != NULL) {
        br->error = err;
        return br;
    }

    if (!hasPropInJSObj(env, params[0], XYZ_PROPS)) {
        br->error = ARG_TYPE_ERR;
        return br;
    }
    
    char *type = getStringValue(env, params[1], MAX_LEN_TYPE);
    if (type == NULL) {
        br->error = CONVERT_ERR;
        return br;
    }

    Validation *validator = getValidationProps(type);
    if (validator == NULL) {
        br->error = OTYPE_TYPE_ERR;
        return br;
    }

    if (validator->output == nil) {
        br->error = INVALID_TYPE;
        return br;
    }

    free(type);
    
    br->color  = params[0];
    br->output = validator->output;
    br->matrix = NULL;
    br->error  = NULL;
    br->clamp  = 0.0;
  
    free(validator);
    getClampOpt(env, obj, br);
    
    return br;
}
